#ident "@(#) space.c 25.x 93/06/13 "
/*      
 *	Copyright (C) 1991-1993 The Santa Cruz Operation, Inc.
 *		All Rights Reserved.
 *	The information in this file is provided for the exclusive use of
 *	the licensees of The Santa Cruz Operation, Inc.  Such users have the
 *	right to use, modify, and incorporate this code into other products
 *	for purposes authorized by the license agreement provided they include
 *	this notice and the associated copyright notice with any such product.
 *	The information in this file is provided "AS IS" without warranty.
 */

#include "sys/paconf.h"
#include "config.h"

/*
 * the number of parallel cards configured in
 */
int pa_n_configured = PA_CNTLS;

/*
 * whether we believe the sdevice configuration for MicroChannel m/cs
 */
int pa_believe_sdev = 0;

/*
 * interrupt type for parallel ports
 */
int pa_int_type = PA_TYPE;

/*
 * the cards configured in.
 */
pa_conf_t pa_boards[PA_CNTLS] = {
#ifdef PA_0
	{ PA_0, PA_0_VECT, (struct pa_regs *)PA_0_SIOA, PA_0_EIOA, PA_0_IPL },
#endif /* PA_0 */
#ifdef PA_1
	{ PA_1, PA_1_VECT, (struct pa_regs *)PA_1_SIOA, PA_1_EIOA, PA_1_IPL },
#endif /* PA_1 */
#ifdef PA_2
	{ PA_2, PA_2_VECT, (struct pa_regs *)PA_2_SIOA, PA_2_EIOA, PA_2_IPL },
#endif /* PA_2 */
};

/*
 * After patime() is called pa_intrrpt_thrshld times without detecting 
 * an interrupt from the port it will set POLL mode in pa_state.
 */
int pa_intrrpt_thrshld = 4;

/*
 * The patune[] array is an array of structures containing:
 *
 *  pa_nmbr			pa port number (don't change here)
 *  pa_vctr			pa port configured vector (don't change here)
 *  pa_addr			pa port configured base address (don't "" "")
 *
 *  - Configurable here - 
 *  pa_maxch			max # characters sent per intr or poll
 *  pa_poll			poll printer every pa_poll clock ticks 
 *  pa_mode_select		1 if printer checks mode select, else 0
 *  pa_sspnd_bfr_strb		n-microsec delay before dropping strobe
 *  pa_sspnd_aftr_strb		n-microsec delay after dropping strobe
 *  pa_sspnd_bfr_bsy_lp		n-microsec before 1st test for busy 
 *  pa_bsy_lp_cnt		number of busy checks before giving up 
 *  pa_sspnd_in_bsy_lp		n-microsec delay after failed busy check
 *  pa_intrrpt_cmn_err		if 1 then cmn_err msg each intr or poll
 *
 * patune[] can be accessed via the PIOC_GET_TUNE and PIOC_SET_TUNE ioctls
 * 
 * After a data character is strobed, the driver checks the port for not busy. 
 * If not busy then another character is sent.  If busy then the driver loops 
 * and checks again.  After pa_bsy_lp_cnt loops it assumes the printer is really
 * busy and exits. 
 *
 * If pa_intrrpt_cmn_err is set to TRUE (1) then a cmn_err message will be 
 * sent to the console whenever the port interrupts or polling is detected.
 * This should remain 0 for production use as it can clutter the console badly.
 */

struct pa_tune pa_tune[PA_CNTLS] = {
#ifdef PA_0
	{ PA_0, PA_0_VECT, (struct pa_regs *)PA_0_SIOA, 
	  150, 1, 1, 2, 2, 2, 12, 1, 0 },
#endif /* PA_0 */
#ifdef PA_1
	{ PA_1, PA_1_VECT, (struct pa_regs *)PA_1_SIOA, 
	  150, 1, 1, 2, 2, 2, 12, 1, 0 },
#endif /* PA_1 */
#ifdef PA_2
	{ PA_2, PA_2_VECT, (struct pa_regs *)PA_2_SIOA, 
	  150, 1, 1, 2, 2, 2, 12, 1, 0 },
#endif /* PA_2 */
};

/*
 * The pacount[] array is an array of structures containing:
 *
 *  pa_nmbr			pa port number (don't change here)
 *  - Configurable here - 
 *  pa_intrrpt_cnt		# times pastart called due to interrupt
 *  pa_intrrpt_chrs		# chars output due to interrupts
 *  pa_poll_cnt			# times pastart called due to polling
 *  pa_poll_chrs		# chars output due to polling
 *  pa_write_cnt		# times pastart called from line discipline
 *  pa_write_chrs		# chars output direct from line discipline
 *  pa_entrd_lp			# times busy loop was entered
 *  pa_exctd_lp_bdy		# times busy loop body was executed
 *  pa_extd_lp_bsy		# times exited loop w/printer still busy
 *
 * pacount[] can be accessed via the PIOC_GET_COUNTS and PIOC_SET_COUNTS ioctls
 * 
 * A reasonable new minimum for pa_bsy_lp_cnt can be obtained by the formula:
 * (pa_exctd_lp_bdy-(pa_bsy_lp_cnt*pa_extd_lp_bsy))/(pa_entrd_lp-pa_extd_lp_bsy)
 * and rounding up.  This formula gives the average number of times the
 * body of the loop is executed per entry of the loop, not counting cases
 * in which the printer remained busy.
 */
struct pa_counts pa_counts[PA_CNTLS] = {
#ifdef PA_0
	{ PA_0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
#endif /* PA_0 */
#ifdef PA_1
	{ PA_1, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
#endif /* PA_1 */
#ifdef PA_2
	{ PA_2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
#endif /* PA_2 */
};
