#ifndef _SYS_PACONF_H
#define _SYS_PACONF_H
#ifdef __STDC__
#pragma comment(exestr, "@(#) paconf.h 25.X 93/06/13 ")
#else
#ident "@(#) paconf.h 25.X 93/06/13 "
#endif
/*
 *	Copyright (C) 1991 The Santa Cruz Operation, Inc.
 *		All Rights Reserved.
 *	The information in this file is provided for the exclusive use of
 *	the licensees of The Santa Cruz Operation, Inc.  Such users have the
 *	right to use, modify, and incorporate this code into other products
 *	for purposes authorized by the license agreement provided they include
 *	this notice and the associated copyright notice with any such product.
 *	The information in this file is provided "AS IS" without warranty.
 */

/*
 *	Parallel I/O Configuration
 */

/*
 * per card
 */
typedef struct pa_conf {
	int pa_unit;			/* unit number */
	int pa_vec;			/* interrupt vector */
	struct pa_regs *pa_addr;	/* base address */
	int pa_eio;			/* end address */
	int pa_ipl;			/* IPL value */
} pa_conf_t;

/*
 * This is the structure used to extract the pa port h/w settings
 * with the PIOCPADMP and PIOCRSTSTAT ioctls.
 */

typedef struct pa_info {
        unsigned data;
        unsigned status;
        unsigned cntrl;
        unsigned short pa_state;
        unsigned short pa_xstate;
} pa_info_t;

/*
 * This is the structure used by the patune[] array which holds the
 * per-port variables for tuning the parallel port.  They are especially
 * useful for tuning ports operating in polling mode.
 */
typedef struct pa_tune {
        int pa_nmbr;            /* pa port number for ref. running crash   */
        int pa_vctr;            /* pa port configured vector for crash     */
        struct pa_regs *pa_addr;/* pa port configured base address f/crash */
        int pa_maxch;           /* max # characters sent per intr or poll  */
        int pa_poll;            /* poll printer every pa_poll clock ticks  */
        int pa_mode_select;     /* 1 if printer checks mode select, else 0 */
        int pa_sspnd_bfr_strb;  /* n-microsec delay before dropping strobe */
        int pa_sspnd_aftr_strb; /* n-microsec delay after dropping strobe  */
        int pa_sspnd_bfr_bsy_lp;/* n-microsec before 1st test for busy     */
        int pa_bsy_lp_cnt;      /* number of busy checks before giving up  */
        int pa_sspnd_in_bsy_lp; /* n-microsec delay after failed busy check*/
        int pa_intrrpt_cmn_err; /* if 1 then cmn_err msg each intr or poll */
} ;

/*
 * This is the structure used by the pacounts[] array holds per-port
 * statistics for use in tuning the parallel port.
 */
typedef struct pa_counts {      /* per-port stats for tuning (esp. polled) */
        int pa_nmbr;            /* pa port number for ref. running crash   */
        int pa_intrrpt_cnt;     /* # times pastart called due to interrupt */
        int pa_intrrpt_chrs;    /* # chars output due to interrupts        */
        int pa_poll_cnt;        /* # times pastart called due to polling   */
        int pa_poll_chrs;       /* # chars output due to polling           */
        int pa_write_cnt;       /* # times pastart called from write(S)    */
        int pa_write_chrs;      /* # chars output directly from write(S)   */
        int pa_entrd_lp;        /* # times busy loop was entered           */
        int pa_exctd_lp_bdy;    /* # times busy loop body was executed     */
        int pa_extd_lp_bsy;     /* # times exited loop w/printer still busy*/
} ;

#define PIOC                    ('P' << 8)
#define PIOCPGLEN               (PIOC | 1)
#define PIOCPADMP               (PIOC | 2)
#define PIOC_GET_TUNE           (PIOC | 3)
#define PIOC_SET_TUNE           (PIOC | 4)
#define PIOC_GET_COUNTS         (PIOC | 5)
#define PIOC_RESET_COUNTS       (PIOC | 6)
#define PIOCRSTSTAT		(PIOC | 7)
#define PIOCSTAT		PIOCPADMP

/* status defines */
#define NBUSY		0x80	/* not busy */
#define NACK		0x40	/* acknowledge */
#define PE		0x20	/* paper empty */
#define SELECT		0x10	/* selected */	
#define NERROR		0x08	/* no error */

/* cntrl defines  */
#define EXTMODE		0x20	/* if set, enable extended mode */
#define IE		0x10	/* interrupt enable */
#define SLCTIN		0x08
#define ENABL		0x04
#define AUTOFEED	0x02	/* auto-line-feed mode */
#define STROBE		0x01	/* strobe a character out */

/* pa_state defines */
#define	PAPRES		0x0001	/* printer present */
#define OFFLINE		0x0002	/* printer offline */
#define INTR		0x0004	/* printer interrupt has occured */
#define POLL		0x0008	/* use poll routine */
#define PATIM_PEND	0x0010	/* patime still pending */
#define PAPOLL_PEND	0x0020	/* poll is running */
#define PROB_POLL	0x0040	/* continue poll */
#define	OPEN_POLL	0x0080	/* open for polling */
#define PA_ISOPEN	0x0100  /* internal ISOPEN */
#define IN_PACLOSE	0x0200	/* in paclose routine */
#define IN_PAPOLL	0x0400	/* in papoll routine */
#define IN_PATIME	0x0800	/* in patime routine */
#define	DO_XLAT		0x1000	/* do dumb printer xlation */

/* pa_xstate xlation state values */
#define	XL_NORM		0	/* normal state */
#define	XL_BS		1	/* esc char after backspace */
#define	XL_TAB		2	/* expand tab */
#define	XL_FF		3	/* expand form-feed */

#ifdef _INKERNEL /* Hide from normal users (especially C++) */

/*
 * the number of parallel cards configured in
 */
extern int pa_n_configured;

/*
 * the cards configured in.
 */
extern pa_conf_t pa_boards[];

/*
 * whether to believe the sdevice configuration info for MCA boxes
 */
extern int pa_believe_sdev;

/*
 * interrupt type for parallel port
 */
extern int pa_int_type;

/*
 * After pa_intrrpt_thrshld 1/4 second periods, if output is still
 * waiting for an interrupt from the printer then it is assumed that the
 * printer or the port is incapable of interrupting and the port is
 * switched to polled mode.
 */
extern int pa_intrrpt_thrshld;

/*
 * This array holds per-port variables for tuning the parallel port.
 * The array is initialized in /etc/conf/pack.d/pa/space.c.
 */
extern struct pa_tune pa_tune[];

/*
 * This array holds per-port statistics for tuning the parallel port
 * using the patune[] variables above.  Also initialized in ../pa/space.c
 */
extern struct pa_counts pa_counts[];

#endif /* _INKERNEL */

#endif /* _SYS_PACONF_H */
